#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['air', 'air.tests']

package_data = \
{'': ['*']}

setup(name='air52',
      version='0.0.1',
      description='Air.',
      author='Yura52',
      author_email='strausmg@gmail.com',
      url='https://github.com/Yura52/air',
      packages=packages,
      package_data=package_data,
      python_requires='>=3.7',
     )
